require 'singleton'
require_relative 'client'
require_relative 'plugins/hmac_auth'

module Kodmin
  # Plugins of Kong.
  class Plugins
    include Singleton
    include Client

    def list_plugins(offset = '')
      params = offset.empty? ? {} : { offset: offset }
      resp = get('/plugins', params)
      resp = JSON.parse(resp)
      plugins = []
      resp['data'].each do |v|
        plugins << plugin_from_hash(v)
      end
      offset_token = nil
      unless resp['next'].nil? || resp['next'].empty?
        offset_token = resp['next'].split('=')[1]
      end
      { plugins: plugins, offset: offset_token }
    end

    def enable_plugin(attributes)
      resp = post('/plugins', attributes)
      resp = JSON.parse(resp)
      plugin_from_hash(resp)
    end

    def disable_plugin(id)
      delete("/plugins/#{id}")
    end

    def plugin_from_hash(attributes)
      case attributes['name']
      when Kodmin::Plugins::HmacAuth::NAME then Kodmin::Plugins::HmacAuth.new(attributes)
      else Kodmin::Plugins::Plugin.new(attributes)
      end
    end
  end
end
