require_relative 'plugin'

module Kodmin
  class Plugins
    # Plugin: hmac-auth
    class HmacAuth < Plugin
      NAME = 'hmac-auth'.freeze

      # HMAC auth's nested configuration.
      class Config
        attr_accessor :hide_credentials, :clock_skew, :anonymous, :validate_request_body, :enforce_headers, :algorithms
        def initialize(attrs = {})
          attrs.each_key do |key|
            setter = "#{key}="
            send(setter, attrs[key]) if respond_to?(setter)
          end
        end

        def to_hash
          hash = {}
          hash[:hide_credentials] = hide_credentials unless hide_credentials.nil?
          hash[:clock_skew] = clock_skew unless clock_skew.nil?
          hash[:anonymous] = anonymous unless anonymous.nil?
          hash[:validate_request_body] = validate_request_body unless validate_request_body.nil?
          hash[:enforce_headers] = enforce_headers unless enforce_headers.nil?
          hash[:algorithms] = algorithms unless algorithms.nil?
          hash
        end
      end

      attr_accessor :config

      def initialize(attrs = {})
        form_hash(attrs)
      end

      def name
        HmacAuth::NAME
      end

      def to_hash
        hash = super
        hash[:config] = config.to_hash unless config.nil?
        hash
      end

      # Assign attributes from Hash.
      # @param attrs [Hash] attributes in Hash object.
      def form_hash(attrs = {})
        attrs.each_key do |key|
          if key.to_sym == :config
            self.config = HmacAuth::Config.new(attrs[key])
          else
            setter = "#{key}=".to_sym
            send(setter, attrs[key]) if respond_to?(setter)
          end
        end
      end
    end
  end
end
